package com.loafle.overflow.crawler.snmp;

import com.loafle.overflow.crawler.Crawler;
import com.loafle.overflow.crawler.snmp.version.SNMPv2c;
import com.loafle.overflow.crawler.snmp.version.SNMPv3;
import org.snmp4j.Snmp;
import org.snmp4j.TransportMapping;
import org.snmp4j.smi.Address;
import org.snmp4j.transport.DefaultUdpTransportMapping;

import java.io.IOException;
import java.util.Map;

/**
 * Created by insanity on 17. 4. 10.
 */
public class SNMPCrawler extends Crawler {

    @Override
    public Object getInternal(Map<String, Object> config) {

        String targetVer = (String)config.get("version");
        Object retObj = null;

        switch (targetVer) {
            case "v2c":
                retObj = getByV2c(config);
                break;
            case "v3":
                retObj = getByV3(config);
                break;
            default:
                new Exception("Unknown SNMP protocol : " + targetVer).printStackTrace();
        }

        return retObj;
    }

    private Object getByV2c(Map<String, Object> config) {
        Snmp snmp = null;
        TransportMapping<? extends Address> transport = null;
        try {
            transport = new DefaultUdpTransportMapping();
            snmp = new Snmp(transport);
            transport.listen();
            SNMPv2c snmpV2 = new SNMPv2c(snmp);

            String ip = (String)config.get("ip");
            String port = (String)config.get("port");
            String community = (String)config.get("community");
            String[] oids = (String[])config.get("oids");
            String method = (String)config.get("method");

            switch (method) {
                case "validate":
                    return snmpV2.validate(ip, port, community);
                case "get":
                    return snmpV2.get(ip, port, community, oids);
                case "walk":
                    return snmpV2.walk(ip, port, community, oids[0]);
                default:
                    new Exception("Unknown SNMP command : " + method).printStackTrace();
            }

        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (snmp != null) {
                try {
                    snmp.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (transport != null) {
                try {
                    transport.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return null;
    }

    private Object getByV3(Map<String, Object> config) {
        Snmp snmp = null;
        TransportMapping<? extends Address> transport = null;

        try {
            transport = new DefaultUdpTransportMapping();
            snmp = new Snmp(transport);
            transport.listen();
            SNMPv3 snmpV3 = new SNMPv3(snmp);

            String ip = (String)config.get("ip");
            String port = (String)config.get("port");
            String user = (String)config.get("user");
            String authType = (String)config.get("authType");
            String authPass = (String)config.get("authPass");
            String privType = (String)config.get("privType");
            String privPass = (String)config.get("privPass");
            String[] oids = (String[])config.get("oids");
            String method = (String)config.get("method");

            switch (method) {
                case "validate":
                    return snmpV3.validate(ip, port, user, authType, authPass, privType, privPass);
                case "get":
                    return snmpV3.get(ip, port, user, authType, authPass, privType, privPass, oids);
                case "walk":
                    return snmpV3.walk(ip, port, user, authType, authPass, privType, privPass, oids[0]);
                default:
                    new Exception("Unknown SNMP command : " + method).printStackTrace();
            }

        } catch (Exception e) {
            new Exception("An Exception happend : " + e.getMessage()).printStackTrace();
        } finally {
            if (snmp != null) {
                try {
                    snmp.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (transport != null) {
                try {
                    transport.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return null;
    }
}